#!/bin/sh
NODE_ROOT='/usr/NX'

COMMAND_ECHO="/bin/echo"
COMMAND_CAT="/bin/cat"
COMMAND_SU="/bin/su"
COMMAND_RM="/bin/rm"
COMMAND_CHOWN="/bin/chown"

if [ $# -ne 6 ];
  then
    $COMMAND_ECHO "NX> 599 Error: Wrong parameters count." && exit 1
fi

TMP_DIR="/tmp"
TMP_FILE="$TMP_DIR/temporary.cookie.$$"
COMMAND_XAUTH="$3"
USERNAME="$4"
DISPLAY="$5"
GLOBALXAUTHORITY="$6"

if [ "x$1" != "xnx" ];
  then
    $COMMAND_ECHO "NX> 599 Error: generatecookie.sh only 'nx' user can generate new cookie by this script" 1>&2
    exit 1
fi

if [ "x$COMMAND_XAUTH" = "x" ];
then
 $COMMAND_ECHO "NX> 599 Error: generatecookie.sh missing command xauth parameter" 1>&2
 exit 1
fi

checkPpid() {

  if [ "x$1" = "x" ];
  then
    $COMMAND_ECHO "Missing pid"
    return 1
  fi
  if [ ! -e /proc/$1/cmdline ];
  then
    $COMMAND_ECHO "Wrong pid: $1"
    return 1
  fi

  command=$($COMMAND_CAT /proc/$1/cmdline)

  $COMMAND_ECHO "$command"
}

command=$(checkPpid $2)

if [ $? != 0 ];
then
  $COMMAND_ECHO "NX> 500 Error: $command"
  exit 1
fi

if [ "$command" != "nxserver-c$NODE_ROOT/bin/nxserver --login" ];
then
  $COMMAND_ECHO "NX> 500 Error: Only the nxserver can run this script: $command.\n";
  exit 1
fi

if [ ! -x $COMMAND_XAUTH ];
then
  $COMMAND_ECHO "NX> 500 Error: Command: $COMMAND_XAUTH doesnt exist."
  exit 1
fi

if [ "x$GLOBALXAUTHORITY" != "x" ];
then
  TMP_FILE="$NODE_ROOT/home/nx/temporary.cookie.$$"
  command=`XAUTHORITY=$GLOBALXAUTHORITY $COMMAND_XAUTH -q -f $TMP_FILE generate :$DISPLAY MIT-MAGIC-COOKIE-1  trusted timeout 120 2>&1`
  if [ "$?" -ne 0 ];
    then
      $COMMAND_ECHO "NX> 500 Error: Cannot generate cookie: $command"
    exit 1
  fi
else
  command=`$COMMAND_SU - $USERNAME -c "$COMMAND_XAUTH -q -f $TMP_FILE generate :$DISPLAY MIT-MAGIC-COOKIE-1  trusted timeout 120 2>&1"`
  if [ "$?" -ne 0 ];
    then
      $COMMAND_ECHO "NX> 500 Error: Cannot generate cookie: $command"
    exit 1
  fi
fi

cookie=`$COMMAND_ECHO list :$DISPLAY | $COMMAND_XAUTH -q -f $TMP_FILE`
if [ "$?" -ne 0 ];
  then
  $COMMAND_ECHO "NX> 500 Error: Cannot retrieve cookie: $!"
  exit 1
fi

ttt=`$COMMAND_CHOWN nx $TMP_FILE`
if [ "$?" -ne 0 ];
  then
  $COMMAND_ECHO "NX> 500 Error: Cannot set ownership of temporary file: $!"
  exit 1
fi

$COMMAND_ECHO "Cookie generated: $cookie in file: $TMP_FILE."

